package me.akito123321.valoStrats.rest.util;

import java.util.UUID;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.InsufficientAuthenticationException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.authentication.dao.AbstractUserDetailsAuthenticationProvider;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.UserDetails;

import com.google.gson.JsonObject;

import me.akito123321.valoStrats.Config;
import me.akito123321.valoStrats.ValoStratsApplication;
import me.akito123321.valoStrats.rest.oauth2.GoogleService;
import okhttp3.OkHttpClient;
import retrofit2.Call;
import retrofit2.Retrofit;
import retrofit2.converter.gson.GsonConverterFactory;

public class GoogleAuthenticationManager extends AbstractUserDetailsAuthenticationProvider {

	@Autowired
	private JDBCUserDetailsService userDetailsService;

	private OkHttpClient.Builder httpClient = new OkHttpClient.Builder();
	private Retrofit retrofit = new Retrofit.Builder()
		.baseUrl("https://cringe-studios.com")
		.addConverterFactory(GsonConverterFactory.create())
		.client(httpClient.build())
		.build();
	private GoogleService service = retrofit.create(GoogleService.class);

	@Override
	protected void additionalAuthenticationChecks(UserDetails userDetails, UsernamePasswordAuthenticationToken authentication) throws AuthenticationException {
	}

	@Override
	protected UserDetails retrieveUser(String username, UsernamePasswordAuthenticationToken authentication) throws AuthenticationException {
		Config config = ValoStratsApplication.config;
		String accessToken = (String) authentication.getCredentials();
		Call<JsonObject> tokenRequest = service.getAuthorizationToken(config.clientId(), config.clientSecret(), accessToken, "authorization_code", config.redirect_url());

		UserDetails details = null;
		try {
			JsonObject response = tokenRequest.execute().body();
			System.out.println(response.toString());

			JsonObject userInfo = service.getUserInfo("Bearer " + response.get("access_token").getAsString()).execute().body();
			System.out.println(userInfo.toString());

			String googleUserId = userInfo.get("sub").getAsString();
			String name = userInfo.get("name").getAsString();

			details = userDetailsService.loadByGoogleId(googleUserId);
			if (details == null) {
				details = userDetailsService.createUser(UUID.randomUUID().toString(), "", googleUserId, name);
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new InsufficientAuthenticationException("Failed to authenticate");
		}

		System.out.println(details);

		return details;
	}

}
