package me.akito123321.valoStrats.rest.util;

import java.util.List;
import java.util.UUID;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;

import me.akito123321.valoStrats.db.repositories.UserRepository;
import me.akito123321.valoStrats.schemas.StratsUser;

public class JDBCUserDetailsService implements UserDetailsService {

	@Autowired
	private UserRepository userRepository;

	private PasswordEncoder passwordEncoder;

	public JDBCUserDetailsService(PasswordEncoder passwordEncoder) {
		this.passwordEncoder = passwordEncoder;
	}

	public StratsUser createUser(String username, String password, String googleUserId, String displayName) {
		StratsUser user = new StratsUser(googleUserId, displayName, List.of(new SimpleGrantedAuthority("ROLE_USER")), username, passwordEncoder.encode(password), true, true, true, true);
		userRepository.save(user);
		return user;
	}

	public String createToken(String username) {
		StratsUser user = userRepository.findById(username).orElse(null);
		if (user == null)
			return null;

		String token = UUID.randomUUID().toString();
		user.getTokens().add(token);

		userRepository.save(user);

		return token;
	}

	@Override
	public StratsUser loadUserByUsername(String username) throws UsernameNotFoundException {
		StratsUser user = userRepository.findById(username).orElse(null);
		if (user == null)
			throw new UsernameNotFoundException("User not found");
		return user;
	}

	public UserDetails loadByGoogleId(String googleId) {
		UserDetails user = userRepository.findByGoogleUserId(googleId).orElse(null);
		return user;
	}

	public StratsUser loadUserByToken(String token) {
		return userRepository.findByTokensIn(List.of(token)).orElse(null);
	}

}
